import { ChangeDetectorRef, Component, OnInit, EventEmitter, Output } from '@angular/core';
import { MessageConfig, MessageData, Message } from '../message-types/bot-messages';
import { MarkdownService } from '../services/markdown.service';
import { WebSocketService } from '../services/websocket.service';

import { config } from '../config';
import { Subject, Observer, BehaviorSubject } from 'rxjs';

@Component({
  selector: 'app-messages',
  templateUrl: './messages.component.html',
  styleUrls: ['./messages.component.scss']
})
export class MessagesComponent implements OnInit {
  @Output() newMessageEvent: EventEmitter<any> = new EventEmitter();
  newMessage: Subject<MessageData> = new Subject<MessageData>();
  showTypingIndicator = false;

  originator = 'avataobot';
  messages: MessageData[] = [];

  constructor(
    private markdownService: MarkdownService,
    private websocketService: WebSocketService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit() {
    this.newMessage.subscribe(
      message => {
        this.showTypingIndicator = true;
        this.writeMessage(message);
        this.newMessageEvent.emit();
    });

    this.websocketService.connect();
    this.websocketService.observeKey<Message>('message.send').subscribe(
      message => this.newMessage.next(message)
    );
    this.websocketService.observeKey<MessageConfig>('message.config').subscribe(
      message => this.configureMessages(message)
    );
  }

  writeMessage(message: MessageData) {
    this.transformMessage(message);
    this.messages.push(message);
    this.changeDetectorRef.detectChanges();
  }

  transformMessage(message: MessageData) {
    message.message = this.convertMarkdownToHTML(message.message);
    if (!message.originator) {
      message.originator = this.originator;
    }
    if (!message.timestamp) {
      message.timestamp = new Date();
    }
    this.showTypingIndicator = message.typing;
  }

  configureMessages(message: MessageConfig) {
    if (message.originator) {
      this.originator = message.originator;
    }
  }

  convertMarkdownToHTML(text: string) {
    return this.markdownService.convertToHtml(text);
  }
}
